#define AUTO_DLLMAIN
#define PI 3.14159
#include <DynRPG/DynRPG.h>
#include <stdlib.h>
#include <sstream>
#include <vector>
#include <time.h>
#include <cmath>
#include <list>

struct Particle{
    RPG::Image* img;
    double size;
    double x, dx, y, dy;
};

class ParticleEffect{
    private:
        std::list<RPG::Image*>images;
        std::list<Particle>particles;
        double dr, dg, db, ds, dA;

    public:

    int amount, size;
    int red, green, blue;
    int rndX, rndY;
    double spd, rndSpd;
    double timeout, delay;
    double angleS, angleE;
    bool mask;
    std::string filename;

    ParticleEffect()
    {
        dr = dg = db = ds = 0;
        red = green = blue = 255;
        timeout = 30;
        amount = 50;
        delay = 0;
        size = 2;
        dA = 8.5;
        spd = 2;
        rndSpd = 2;
        rndX = rndY = 0;
        angleS = 0;
        angleE = 2*PI;
        mask = false;
        filename = "DynPlugins/Particle.PNG";
    }

    void setTimeout(double t)
    {
        dA = 255/(t - delay);
        timeout = t;
    }

    void setAngle(double a1, double a2)
    {
        angleS = a1*PI/180;
        angleE = (a2 - a1)*PI/180;
    }

    void setGrowth(int s, int newSize)
    {
        size = s;
        ds = (newSize - size)/timeout;
    }

    void colorFade(int r = 0, int g = 0, int b = 0)
    {
        dr = ((double)r - red)/timeout;
        dg = ((double)g - green)/timeout;
        db = ((double)b - blue)/timeout;
    }

    void newBurst(int x, int y)
    {
        images.push_back(RPG::Image::create());
        images.back()->colorControl2->red = red;
        images.back()->colorControl2->green = green;
        images.back()->colorControl2->blue = blue;
        images.back()->useMaskColor = mask;
        images.back()->loadFromFile(filename);
        for(int i = 0; i<amount; i++)
        {
            double rnd = angleS + ((float)rand()/(float)RAND_MAX)*angleE;
            double newSpd = (spd + rndSpd*((float)rand()/(float)RAND_MAX));
            Particle pa;
            pa.x = x + 2*rndX*((float)rand()/(float)RAND_MAX) - rndX;
            pa.y = y + 2*rndY*((float)rand()/(float)RAND_MAX) - rndY;
            pa.dx = newSpd*std::cos(rnd);
            pa.dy = newSpd*std::sin(rnd);
            pa.img = images.back();
            pa.size = size;
            particles.push_back(pa);
        }
    }

    void newHeart(int x, int y)
    {
        images.push_back(RPG::Image::create());
        images.back()->colorControl2->red = red;
        images.back()->colorControl2->green = green;
        images.back()->colorControl2->blue = blue;
        images.back()->useMaskColor = mask;
        images.back()->loadFromFile(filename);
        for(int i = 0; i<amount; i++)
        {
            double rnd = ((float)rand()/(float)RAND_MAX)*2*PI;
            Particle pa;
            pa.x = x;
            pa.y = y;
            pa.dx = spd*(16*std::pow(std::sin(rnd),3));
            pa.dy = -spd*(13*cos(rnd)-5*std::cos(2*rnd)-2*std::cos(3*rnd)-std::cos(4*rnd));
            pa.img = images.back();
            pa.size = size;
            particles.push_back(pa);
        }
    }

    void drawBurst()
    {
        if(particles.size() == 0)
            return;
        std::list<RPG::Image*>::iterator imgItr = images.begin();
        while(imgItr != images.end())
        {
            (*imgItr)->colorControl1->red += dr;
            (*imgItr)->colorControl1->green += dg;
            (*imgItr)->colorControl1->blue += db;
            if(delay <= 0)
            {
                (*imgItr)->alpha -= dA;
                if((*imgItr)->alpha <= 0)
                {
                    for(int i = 0; i<amount; i++)
                        particles.pop_front();
                    imgItr++;
                    RPG::Image::destroy(images.front());
                    images.pop_front();
                    continue;
                }
                (*imgItr)->loadFromFile(filename);
            }
            else
            {
                delay--;
            }
            imgItr++;
        }
        std::list<Particle>::iterator itr = particles.begin();
        while(itr != particles.end())
        {
            (*itr).x += (*itr).dx;
            (*itr).y += (*itr).dy;
            (*itr).size += ds;
            RPG::screen->canvas->drawStretched((*itr).x - RPG::map->getCameraX(), (*itr).y - RPG::map->getCameraY(), (int)(*itr).size, (int)(*itr).size, (*itr).img);
            itr++;
        }
    }

    void killParticles()
    {
        particles.clear();
        for(int i = images.size(); i > 0; i--)
        {
            RPG::Image::destroy(images.front());
            images.pop_front();
        }
    }
};

std::vector<ParticleEffect*> effects;
bool draw = true;

void onDrawScreen()
{
    if(draw)
        for(int i = effects.size()-1; i >= 0; i--)
        {
            effects[i]->drawBurst();
        }
}

bool onComment( const char* text,
                const RPG::ParsedCommentData* 	parsedData,
                RPG::EventScriptLine* 	nextScriptLine,
                RPG::EventScriptData* 	scriptData,
                int 	eventId,
                int 	pageId,
                int 	lineId,
                int* 	nextLineId )
{
    std::string s = parsedData->command;
    std::stringstream ss;
    if(s.compare("effect_burst")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->newBurst(parsedData->parameters[1].number,
                                    parsedData->parameters[2].number);
        }
        return false;
    }
    if(s.compare("effect_heart")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->newHeart(parsedData->parameters[1].number,
                                    parsedData->parameters[2].number);
        }
        return false;
    }
    if(s.compare("new_effect")==0)
    {
        effects.push_back(new ParticleEffect());
        return false;
    }
    if(s.compare("effect_colorfade")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->colorFade(parsedData->parameters[1].number,
                                     parsedData->parameters[2].number,
                                     parsedData->parameters[3].number);
        }
        return false;
    }
    if(s.compare("effect_color")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->red = parsedData->parameters[1].number;
           effects[index]->green = parsedData->parameters[2].number;
           effects[index]->blue = parsedData->parameters[3].number;
        }
        return false;
    }
    if(s.compare("effect_amount")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->amount = parsedData->parameters[1].number;
        }
        return false;
    }
    if(s.compare("effect_random")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->rndX = parsedData->parameters[1].number;
           effects[index]->rndY = parsedData->parameters[2].number;
        }
        return false;
    }
    if(s.compare("effect_angle")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->setAngle(parsedData->parameters[1].number,
                                    parsedData->parameters[2].number);
        }
        return false;
    }
    if(s.compare("effect_growth")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->setGrowth(parsedData->parameters[1].number,
                                     parsedData->parameters[2].number);
        }
        return false;
    }
    if(s.compare("effect_speed")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->spd = parsedData->parameters[1].number;
           effects[index]->rndSpd = parsedData->parameters[2].number;
        }
        return false;
    }
    if(s.compare("effect_timeout")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->setTimeout(parsedData->parameters[1].number);
        }
        return false;
    }
    if(s.compare("effect_delay")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->delay = parsedData->parameters[1].number;
        }
        return false;
    }
    if(s.compare("effect_file")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->filename = parsedData->parameters[1].text;
           std::string ps = parsedData->parameters[2].text;
           if(ps.substr(0,4).compare("true")==0)
           {
               effects[index]->mask = true;
           }
        }
        return false;
    }
    if(s.compare("effect_kill")==0)
    {
        int index = parsedData->parameters[0].number;
        if(index >= 0 && index < effects.size())
        {
           effects[index]->killParticles();
        }
        return false;
    }
    if(s.compare("stop")==0)
    {
        draw = false;
        return false;
    }
    if(s.compare("start")==0)
    {
        draw = true;
        return false;
    }
    return true;
}

bool onStartup(char* pluginName)
{
    srand(time(NULL));
    return true;
}
